function [zk, BlockSpectraOut, history]=PEBS2TV(A,b,BlockSizes,lambdaVec,xiVec,alphaVec,alphak,rho,ell,zstart)
% Fast ADMM Solver for the optimization problem,
% minimize_x ||Ax-b||_2^2+lambda*||x||_1+sum_k=1^d alpha_k ||x_[k]||_2
%           + xi ||x||_TV
% where x=[x_[1]^T, ..., x_[d]^T]] and each x_[k] is a complex column
% vector in C^n_k.
%
%      INPUT
%
%      A          - In C^(N x M) the dictionary, or matrix of explanatory
%                   variables.
%
%      b          - In C^n is the data, assumed to be a multi-pitch signal
%                   observed in noise.
%
%      alphaVec      - In [0,inf], sets the weighting of the block ell_2 norm.
%
%      lambdaVec     - In [0,inf], sets the weighting of the ell_1 norm.
%
%      xiVecVec      - In [0,inf], sets the weighting of the ell_1 norm.
%
%      If the weighting parameters are vector they are applied pointwise, e.g.
%      for reweigthing.
%
%      BlockSizes - In N^d, Size of each block, i.e., [n_1, n_2, ... n_d].
%
%      OUTPUT
%
%      zk               - Estimate of x^*, the vector that solves the
%                         minization problem.
%
%      BlockSpectraOut  - vector containing 2-norm of each block in zk,
%                          i.e., [||z_[1]||_2, ..., ||z[d]||_2].
%

[N,M]=size( A);
noBlocks=length(BlockSizes);
sqrtBlockSizes=sqrt(BlockSizes)';
CumSumBlockSizes=[0, cumsum(BlockSizes)];
mu=2;
uk1=zeros(N,1);
uk2=zeros(M,1);
uk3=zeros(M-1,1);
dk1=zeros(N,1);
dk2=zeros(M,1);
dk3=zeros(M-1,1);
zk=zeros(M,1);
C=[-1,zeros(1,M-2)]';
R=[-1, 1, zeros(1,M-2)];
F=toeplitz(C,R);
C=[3, -1, zeros(1,M-2)]';
R=[3, -1, zeros(1,M-2)];
FtFpI=toeplitz(C,R);
FtFpI(1,1)=2;
FtFpI(end,end)=2;
FtFpI=sparse(FtFpI);
[Lp, Up,P] = factor(A,F, 1);

warmstart=1;
if warmstart==1
    uk1=A*zstart;
    uk2=zstart;
    uk3=diff(zstart);
end
noIter=1999;
C1=CumSumBlockSizes;
QUIET=1;
if ~QUIET
    fprintf('%3s\t%1s\t%10s\t%10s\t%10s\t%10s\t%10s\n', 'iter', ...
        'r norm', 'eps pri', 's norm', 'eps dual', 'objective','mu');
end
BlockSpectraOut=zeros(noBlocks, length(lambdaVec));
for lambiter=1:length(lambdaVec);
    lambda=lambdaVec(lambiter);
    alpha=alphaVec(lambiter);
    for i=1:noIter
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % zk update
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Ftuk3pdk3=diff( uk3+dk3);
        Ftuk3pdk3=[-uk3(1)-dk3(1);-Ftuk3pdk3;-uk3(end)-dk3(end)];
        Gq= A'*( uk1+dk1)+( uk2+dk2)+Ftuk3pdk3;
        IpFFTinvGq=FtFpI \ (Gq) ;
        zk=IpFFTinvGq- ( FtFpI \ (A'*(Up\(Lp\(P*(A*IpFFTinvGq))))));
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Store old uk1-3, update Azk and zk2, zk3 variables
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        uk1old=uk1;
        uk2old=uk2;
        uk3old=uk3;
        Azk=A*zk;
        zk2=zk; 
        zk3=diff(zk); 
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Over-relaxation, can speed up convergence a factor 2-5
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Azk=alphak*Azk+(1-alphak)*uk1;
        zk2=alphak*zk2+(1-alphak)*uk2;
        zk3=alphak*zk3+(1-alphak)*uk3;
        
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Update uk1, i.e., the ell_1 or ell_2 part of u
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if ell==2
            uk1=(b+mu*(Azk-dk1))/(1+mu);
        end
        if ell==1
            uk1=softthreshold(Azk-dk1-b,1/mu,1)+b;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Update uk2, i.e., the block ell-2 and ell-1 part of u
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        uk2tilde=softthreshold(zk2-dk2,lambda/mu,1);
        uk2=blockthreshold(uk2tilde,0,alpha,mu,C1,sqrtBlockSizes,M);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Update uk3, i.e., the TV part of u
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        uk3=softthreshold(zk3-dk3,xiVec/mu,1);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %Update dkX, i.e., the dual variables
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        dk1=dk1-(Azk-uk1);
        dk2=dk2-(zk2-uk2);
        dk3=dk3-(zk3-uk3);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Convergence check and update of penalty factor mu
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Ftuk3muk3old=diff( uk3-uk3old);
        Ftuk3muk3=[-uk3(1)+uk3old(1);-Ftuk3muk3old;-uk3(end)+uk3old(end)];
        sk=mu*(A'*(uk1-uk1old)+uk2-uk2old+Ftuk3muk3);
        rk= [Azk-uk1;zk2-uk2;zk3-uk3];
        
        if (norm(rk) > rho*norm(sk))
            mu=2*mu;
            dk1=dk1/2;
            dk2=dk2/2;
            dk3=dk3/2;
        elseif (norm(sk)>rho*norm(rk))
            mu=mu/2;
            dk1=dk1*2;
            dk2=dk2*2;
            dk3=dk3*2;
        end
        if ell==2
            ABSTOL=1e-4;
            RELTOL=1e-5;
        end
        if ell==1
            ABSTOL=1e-4;
            RELTOL=1e-5;
        end
        
        history.mu(i)=mu;
        history.r_norm(i)  = norm(rk);
        history.s_norm(i)  = norm(sk);
        normzk2=sum(abs(zk).^2);
        normAzk=sum(abs(Azk).^2);
        normGzk=sqrt(normzk2*2+normAzk);
        history.eps_pri(i) = sqrt(3*M)*ABSTOL + RELTOL*max(normGzk, norm(-[uk1;uk2;uk3]));
        Ftdk3tilde=diff( dk3);
        Ftdk3=[-dk3(1);-Ftdk3tilde;-dk3(end)];
        history.eps_dual(i)= sqrt(2*M+N)*ABSTOL + RELTOL*norm(mu*[A.'*dk1;dk2;Ftdk3]);
        
        %
        if ~QUIET
            fprintf('%3d\t%10.4f\t%10.4f\t%10.4f\t%10.4f\t%10.2f\t%10.2f\n', i, ...
                history.r_norm(i), history.eps_pri(i), ...
                history.s_norm(i), history.eps_dual(i), history.mu(i));
        end
        
        if (history.r_norm(i) < history.eps_pri(i) && ...
                history.s_norm(i) < history.eps_dual(i))
            break;
        end
        
    end
    BlockSpectraOut=blockspectra(CumSumBlockSizes(2:end), zk);
end
end

function xout=softthreshold(xin,lambda,p)
if p == 1
    maxf=max(abs(xin)-lambda,0);
    xout=xin.*maxf./(lambda+maxf);
end
if p == 2
    norm_xin=norm(xin);
    maxf=max(norm_xin-lambda,0);
    xout=xin*maxf/(lambda+maxf);
end
end


function [L, U, P] = factor(A,F, rho)
[m, n]=size(A);
[L, U, P] = lu( speye(m) + 1/rho*(A*(sparse(eye(n)+F'*F)\A')) );
L = sparse(L);
U = sparse(U);
P=sparse(P);
end


function p = blockspectra(cum_part, x)
start_ind = 1;
for i = 1:length(cum_part),
    sel = start_ind:cum_part(i);
    p(i) = norm(x(sel));
    start_ind = cum_part(i) + 1;
end
end

function uk3=blockthreshold(zk3,dk3,alpha,mu,C1,sqrtBlockSizes,M)
zk3Minusdk3=zk3-dk3;
pow=abs(zk3Minusdk3).^2;
cumsumpow=cumsum(pow);
blockpow=cumsumpow(C1(2:end));
blockpow=[sqrt(blockpow(1)); sqrt(diff(blockpow))];
maxFV=max(blockpow-sqrtBlockSizes*alpha/mu,0);
shrinkfactor=maxFV./(maxFV+sqrtBlockSizes*alpha/mu);
uk3=zeros(M,1);
ind = find(shrinkfactor);
if isempty(ind)==0
    for k=1:length(ind)
        j=ind(k);
        uk3((C1(j)+1):C1(j+1))=zk3Minusdk3((C1(j)+1):C1(j+1))*shrinkfactor(j);
    end
end
end
